//---------------------------------------------------------------------------

#include <vcl.h>
#include "ITSSkinF.h"
#include "ITSUtilF.h"
#include "AppGlobalF.h"
#include "ITSLangTransF.h"
#include "ITSMapF.h"
#include "WindowMsgF.h"
#include "ITS_OPLibF.h"
#pragma hdrstop

#include "CRSM100MF.h"
#include "VDSM101MF.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "cxButtons"
#pragma link "cxClasses"
#pragma link "cxContainer"
#pragma link "cxControls"
#pragma link "cxCustomData"
#pragma link "cxData"
#pragma link "cxDataStorage"
#pragma link "cxDropDownEdit"
#pragma link "cxEdit"
#pragma link "cxFilter"
#pragma link "cxGraphics"
#pragma link "cxGrid"
#pragma link "cxGridCustomTableView"
#pragma link "cxGridCustomView"
#pragma link "cxGridLevel"
#pragma link "cxGridTableView"
#pragma link "cxGroupBox"
#pragma link "cxLabel"
#pragma link "cxLookAndFeelPainters"
#pragma link "cxLookAndFeels"
#pragma link "cxMaskEdit"
#pragma link "cxSpinEdit"
#pragma link "cxSplitter"
#pragma link "cxStyles"
#pragma link "cxTextEdit"
#pragma link "dxSkinBlue"
#pragma link "dxSkinsCore"
#pragma link "dxSkinscxPCPainter"
#pragma link "FRAME_OpenMapF"
#pragma link "FRAME_CrsListF"
#pragma link "dxSkinBlack"
#pragma link "dxSkinMcSkin"
#pragma resource "*.dfm"

TCRSM100M *CRSM100M = NULL;
//---------------------------------------------------------------------------
void TCRSM100M_MapCallback(TObject *ASender, const String AFunc, const String AArgs, const bool AFirst)
{
    try
    {
        if (!CRSM100M) return;
        CRSM100M->OpenMapCallback(ASender, AFunc, AArgs, AFirst);
    }
    catch(Exception &exception)
    {
    }
}
//---------------------------------------------------------------------------

__fastcall TCRSM100M::TCRSM100M(TComponent* Owner)
    : TForm(Owner)
{
    LangTrans->Translate(this, ITSDb_GetConnection());
    ITSSkin_Load(this);
    CMM_LoadForm(g_sFormsDir, this);

    FUpdate= false;
    FTitle = Caption;

    FRAMECrsList1->PnlTop->Visible = false;
    FRAMECrsList1->PnlBottom->Visible = false;
    FRAMECrsList1->ColumnSel->Visible = false;

    m_pGDC = FRAMECrsList1->TvList->DataController;

    FLayerIdx = LyrTp_Crs;
    FRAMEOpenMap1->LayerIdx = FLayerIdx;
    FRAMEOpenMap1->MapCallbackFunc = TCRSM100M_MapCallback;
    FRAMEOpenMap1->ChkShowNmbr->Caption = "교차로 ID";

    EditMode = false;
}
//---------------------------------------------------------------------------

void __fastcall TCRSM100M::CommClose()
{
    try
    {
        CMM_SaveForm(g_sFormsDir, this);
    }
    catch(...)
    {
    }
}
//---------------------------------------------------------------------------

void __fastcall TCRSM100M::FormShow(TObject *Sender)
{
    Refresh();

    FRAMEOpenMap1->OpenMap();

    CrossManager->FLists.Lock();
    try
    {
        FOR_STL(TCross*, pObj, CrossManager->FLists)
        {
            if (pObj->DEL_YN == "Y") continue;
            pObj->FData3 = NULL;
        }
    }
    __finally
    {
        CrossManager->FLists.UnLock();
    }

    FormInit();
    ChangeEditMode(1);
    TmrShow->Enabled = true;
}
//---------------------------------------------------------------------------

void __fastcall TCRSM100M::FormInit()
{
	EdNmbr->Text = "";
    EdId->Text = "";
	EdNm->Text = "";

	//CbType->ItemIndex = 0;

    SePosX->Value = 0;
    SePosY->Value = 0;

    EdCtlrIp->Text = "";
    EdCmncPort->Text = "";

    Se_GROUP_NO->Value       = 1;
    Se_LOCAL_NO->Value       = 1;
    Se_TRAF_CLCT_CYCL->Value = 60;
    Se_STTS_CLCT_CYCL->Value = 60;

    Ed_ISTL_LCTN_NM->Text = "";
    ISTL_LCTN_ADDR->Text = "";

    FNewDb = false;
}
//---------------------------------------------------------------------------

void __fastcall TCRSM100M::TmrShowTimer(TObject *Sender)
{
    TmrShow->Enabled = false;
    FRAMECrsList1->UpdateList();
}
//---------------------------------------------------------------------------

void __fastcall TCRSM100M::BtnCloseClick(TObject *Sender)
{
    Close();
}
//---------------------------------------------------------------------------

void __fastcall TCRSM100M::FormClose(TObject *Sender, TCloseAction &Action)
{
    CommClose();
    CRSM100M = NULL;
    Action = caFree;
}
//---------------------------------------------------------------------------

void __fastcall TCRSM100M::FormCloseQuery(TObject *Sender, bool &CanClose)
{
    if (FUpdate)
    {
        POST_MSG(Application->MainForm->Handle, WM_DATABASE_REFRESH, WP_MSG_14, 0);
    }

    FRAMECrsList1->OnCloseQuery(CanClose);
    FRAMEOpenMap1->OnCloseQuery(CanClose);
}
//---------------------------------------------------------------------------

void __fastcall TCRSM100M::DisplayInfo(TCross* AObj, bool AMoveObj/*=true*/)
{
    FormInit();

    if (!AObj) return;

    if (AMoveObj)
    {
        sJs.printf(L"selectLayerObject(%d, '%s', %d)", FLayerIdx, AObj->CROSS_ID.c_str(), FRAMEOpenMap1->MapZoom);
        FRAMEOpenMap1->ExecScript(sJs);
    }
	EdNmbr->Text  = AObj->CROSS_ID;
    EdId->Text = AObj->CROSS_ID;
	EdNm->Text = AObj->CROSS_NM;

	//APP_SetCode(CbType, AObj->VDS_TYPE_CD);

    SePosX->Value = AObj->X_CRDN;
    SePosY->Value = AObj->Y_CRDN;

    EdType->Text  = AObj->CROSS_TYPE;

//    FRAMEGisMap1->SelectObjectByPos(SePosX->Value, SePosY->Value, true);

    ChangeEditMode(3);
}
//---------------------------------------------------------------------------

void __fastcall TCRSM100M::UpdateIcon()
{
    if (!FRAMEOpenMap1->MapStart) return;

    sJs.printf(L"removeLayerObject(%d)", FLayerIdx);
    FRAMEOpenMap1->ExecScript(sJs);

    String arrObj = "";
    String sTemp;

    CrossManager->FLists.Lock();
    try
    {
        FOR_STL(TCross*, pObj, CrossManager->FLists)
        {
            if (pObj->DEL_YN == "Y") continue;

            if (arrObj != "") arrObj += "|";

            sTemp.printf(L"%s,%s,%s,%.6f,%.6f,0,0,0,0", pObj->CROSS_ID.c_str(), pObj->CROSS_ID.c_str(), pObj->CROSS_NM.c_str(), pObj->X_CRDN, pObj->Y_CRDN);
            arrObj += sTemp;
        }
    }
    __finally
    {
        CrossManager->FLists.UnLock();
        if (arrObj != "")
        {
            sJs.printf(L"addFacility(%d, '%s')", FLayerIdx, arrObj.c_str());
            FRAMEOpenMap1->ExecScript(sJs);
        }
    }
}
//---------------------------------------------------------------------------

void __fastcall TCRSM100M::BtnEditClick(TObject *Sender)
{
    String sId = EdNmbr->Text;
    TCross *pObj = CrossManager->FLists.Find(sId);

    FNewDb = false;

    if (BtnEdit->Caption == FrmLang->lblEdit->Caption)//"편집")
    {
        //편집모드 시작
        ChangeEditMode(0);
        if (pObj)
        {
            //시설물아이콘을 편집아이콘으로 변경
            //function editFacilityObject(ALyrIdx, ANmbr, AComm(0:오류-편집,1:정상), AEdit(편집모드:1-편집,0-편집취소))
            sJs.printf(L"editFacilityObject(%d, '%s', 1, 1)", FLayerIdx, pObj->CROSS_ID.c_str());
            FRAMEOpenMap1->ExecScript(sJs);
        }
    }
    else
    {
        //편집모드 취소
        ChangeEditMode(1);
        if (pObj)
        {
            //원래 정보를 화면에 표출
            CMM_SetGridRow(FRAMECrsList1->CxList, FRAMECrsList1->TvList, sId, FRAMECrsList1->Column01->Index);
            DisplayInfo(pObj, false);

#if 0
            //지도시설물아이콘을 원래아이콘으로 변경
            sJs.printf(L"editFacilityObject(%d, '%s', 0, 0)", FLayerIdx, pObj->CROSS_ID.c_str());
            FRAMEOpenMap1->ExecScript(sJs);
#else
            //추가 모드에서 취소하는 경우 지도에 신규로 추가한 객체를 찾아서 삭제해 주는것 보다
            //새롭게 다시 그리자
            UpdateIcon();
#endif
        }
        else
        {
            //객체가 없는 경우는 추가 모드에서 취소한 경우임
            sJs.printf(L"removeFacilityObject(%d, '%s')", FLayerIdx, sId.c_str());
            FRAMEOpenMap1->ExecScript(sJs);
        }
    }
}
//---------------------------------------------------------------------------

void __fastcall TCRSM100M::ChangeEditMode(int AEdit)
{
    bool bEnabled = false;
    switch(AEdit)
    {
    case 0: //편집버튼클릭
        {
            BtnEdit->Enabled = true;
            BtnInsert->Enabled = false;
            BtnDelete->Enabled = false;
            BtnSave->Enabled = true;

            BtnEdit->Caption = FrmLang->lblCancel->Caption;//"취소";
            BtnEdit->Hint    = FrmLang->lblCancel->Hint;//데이터 편집 취소";
            EditMode = true;
        }
        break;
    case 1: //편집[취소]버튼클릭
        {
            FormInit(); //화면을 모두 지운다.
            BtnEdit->Enabled = false;
            BtnInsert->Enabled = true;
            BtnDelete->Enabled = false;
            BtnSave->Enabled = false;

            BtnEdit->Caption = FrmLang->lblEdit->Caption;//"편집";
            BtnEdit->Hint    = FrmLang->lblEdit->Hint;//데이터 편집";
            //bEnabled = true;
            EditMode = false;
        }
        break;
    case 2: //추가
        {
            BtnEdit->Enabled = true;
            BtnInsert->Enabled = false;
            BtnDelete->Enabled = false;
            BtnSave->Enabled = true;

            BtnEdit->Caption = FrmLang->lblCancel->Caption;//"취소";
            BtnEdit->Hint    = FrmLang->lblCancel->Hint;//데이터 편집 취소";
            EditMode = true;
        }
        break;
    case 3: //수정모드
        {
            BtnEdit->Enabled = true;
            BtnInsert->Enabled = true;
            BtnDelete->Enabled = true;
            BtnSave->Enabled = false;

            BtnEdit->Caption = FrmLang->lblEdit->Caption;//"편집";
            BtnEdit->Hint    = FrmLang->lblEdit->Hint;//데이터 편집";
            bEnabled = true;
            EditMode = false;
        }
        break;
    }

    if (AEdit == 0 || AEdit == 2)
    {
        FRAMECrsList1->CxList->Enabled = false;
    }
    else
    {
        FRAMECrsList1->CxList->Enabled = true;
    }

    PnlEdit->Enabled = EditMode;
}
//---------------------------------------------------------------------------


void __fastcall TCRSM100M::BtnInsertClick(TObject *Sender)
{
    int nMsgType = MB_OK|MB_ICONERROR|MB_APPLMODAL;

	String sId = InputBox("교차로 ID",
                          "신규 교차로 ID를 입력하세요.",
                          "최대10자리" );

    sId = sId.Trim();
	if (sId == "최대10자리" || sId == "")
	{
		return;
	}
	if (AnsiString(sId).Length() > 10)
	{
        Application->MessageBox(L"교차로 ID는 10Byte를 초과할 수 없습니다.",
                                FTitle.c_str(), nMsgType);
		return;
	}

    TCross *pDb = (TCross *)CrossManager->FLists.Find(sId);
    if (pDb)
    {
        String sErr = "교차로 ID [" + sId + "]는 이미 사용 중 입니다.";
        Application->MessageBox(sErr.c_str(), FTitle.c_str(), nMsgType);
        return;
    }

    FormInit();

    EdNmbr->Text  = sId;
    EdId->Text    = EdNmbr->Text;
    SePosX->Value = FRAMEOpenMap1->CenterX;
    SePosY->Value = FRAMEOpenMap1->CenterY;

    //지도에 새로운 시설물 아이콘 추가하면서 편집모드로 변경
    String arrObj = "";
    arrObj.printf(L"%s,New,New,%.6f,%.6f,1", sId.c_str(), FRAMEOpenMap1->CenterX, FRAMEOpenMap1->CenterY);
    sJs.printf(L"addFacilityObject(%d, '%s')", FLayerIdx, arrObj.c_str());
    FRAMEOpenMap1->ExecScript(sJs);

    ChangeEditMode(2);
    FNewDb = true;
}
//---------------------------------------------------------------------------

void __fastcall TCRSM100M::BtnDeleteClick(TObject *Sender)
{
    if (!FRAMEOpenMap1->MapStart) return;

    int nIdx = m_pGDC->FocusedRecordIndex;
    if (nIdx < 0) return;

    String sId = EdNmbr->Text;

    if (sId.IsEmpty())
    {
        Application->MessageBox(lblText2->Caption.c_str(),//L"삭제할 VDS를 목록에서 선택하세요.\r\n목록에서 더블클릭하여 선택합니다.",
                                FTitle.c_str(), MB_OK|MB_ICONERROR|MB_APPLMODAL);
        return;
    }

    String sMsg;
    sMsg = lblText3->Caption + ": " + sId;//"VDS번호: " + sId;
    sMsg+= "\r\n" + lblText4->Caption;//"선택하신 VDS 정보를 삭제하시겠습니까?";

    if (Application->MessageBox(sMsg.c_str(), FTitle.c_str(), MB_YESNO|MB_ICONQUESTION|MB_APPLMODAL) != IDYES) return;

	TCross *pObj = CrossManager->FLists.Find(sId);

	if (pObj)
	{
		if (CrossManager->DeleteCross(pObj))
		{
            pObj->DEL_YN = "Y";
            FRAMECrsList1->UpdateList();
            UpdateIcon();
		}

        FUpdate = true;

        Application->MessageBox(lblText5->Caption.c_str(),//L"VDS 정보삭제\r\nVDS 정보를 삭제 하였습니다.",
                                FTitle.c_str(), MB_OK|MB_ICONINFORMATION|MB_APPLMODAL);

        FormInit();
        ChangeEditMode(1);

        UpdateIcon();
	}
}
//---------------------------------------------------------------------------

void __fastcall TCRSM100M::BtnSaveClick(TObject *Sender)
{
	if (EdId->Text.Trim().IsEmpty() ||
        EdNm->Text.Trim().IsEmpty() ||
        SePosX->Value == 0              ||
        SePosY->Value == 0              ||
        EdType->Text.Trim().IsEmpty() )
	{
        Application->MessageBox(FrmLang->lblNecessary->Caption.c_str(),//L"★항목은 필수 입력 항목입니다.\r\n모두 입력후 진행해 주세요.",
                                FTitle.c_str(), MB_OK|MB_ICONERROR|MB_APPLMODAL);
		return;
	}

    String sNmbr = EdId->Text;
    //int    nNmbr = sNmbr.ToInt();
    String sId   = EdId->Text.Trim();

#if 0
    CrossManager->FLists.Lock();
    FOR_STL(TCross*, pTmpObj, CrossManager->FLists)
    {
        if (pTmpObj->DEL_YN == "Y") continue;
        if (pTmpObj->CROSS_ID == sNmbr) continue; //자기자신

        if (pTmpObj->CROSS_ID == sNmbr)
        {
            Application->MessageBox(lblText8->Caption.c_str(), FTitle.c_str(), MB_OK|MB_ICONERROR|MB_APPLMODAL);
            CrossManager->FLists.UnLock();
            return;
        }
    }
    CrossManager->FLists.UnLock();
#endif

    TCross *pObj;
    if (FNewDb)
    {
        CrossManager->FLists.Lock();
        FOR_STL(TCross*, pTmpObj, CrossManager->FLists)
        {
            if (pTmpObj->DEL_YN == "Y") continue;

            if (pTmpObj->CROSS_ID == sId)
            {
                Application->MessageBox(lblText8->Caption.c_str(), FTitle.c_str(), MB_OK|MB_ICONERROR|MB_APPLMODAL);
                CrossManager->FLists.UnLock();
                return;
            }
        }
        CrossManager->FLists.UnLock();

        pObj = new TCross();
    }
    else
        pObj = CrossManager->FLists.Find(sNmbr);

    if (pObj)
    {
        //pObj->CROSS_ID = sNmbr;
        pObj->CROSS_ID   = sId;
        pObj->CROSS_NM   = EdNm->Text.Trim();

        pObj->ZOOM_LEVEL     = 0;
        pObj->LOCAL_GROUP_ID = "";
        pObj->CROSS_TYPE     = EdType->Text.Trim();

        pObj->X_CRDN = SePosX->Value;
        pObj->Y_CRDN = SePosY->Value;


        pObj->Completed    = true;
        pObj->MEM_DEL      = "N";

        bool bResult;
        if (FNewDb)
        {
            pObj->DEL_YN = "N";
            bResult = CrossManager->InsertCross(pObj);
            if (bResult)
            {
                CrossManager->FLists.Push(sNmbr, pObj);
            }
		}
        else
		{
            bResult = CrossManager->UpdateCross(pObj);
		}

        if (bResult)
        {
            Application->MessageBox(lblText9->Caption.c_str(),//L"VDS 정보저장\r\nVDS 정보를 저장 하였습니다.",
                                    FTitle.c_str(), MB_OK|MB_ICONINFORMATION|MB_APPLMODAL);

            FUpdate = true;

            FRAMECrsList1->UpdateList();
            UpdateIcon();
            CMM_SetGridRow(FRAMECrsList1->CxList, FRAMECrsList1->TvList, sNmbr, FRAMECrsList1->Column01->Index);
            DisplayInfo(pObj, true);
        }
    }
}
//---------------------------------------------------------------------------

void __fastcall TCRSM100M::FRAMECrsList1TvListCellDblClick(TcxCustomGridTableView *Sender,
          TcxGridTableDataCellViewInfo *ACellViewInfo, TMouseButton AButton,
          TShiftState AShift, bool &AHandled)
{
	if (!ACellViewInfo) return;
	int nRow = FRAMECrsList1->TvList->DataController->FocusedRecordIndex;
	if( nRow <= -1 )
		return;

    int nMemPtr = FRAMECrsList1->TvList->DataController->Values[nRow][FRAMECrsList1->Column99->Index];
    TCross* pObj = (TCross*)nMemPtr;
    DisplayInfo(pObj, true);
}
//---------------------------------------------------------------------------

void __fastcall TCRSM100M::Ed_VDS_PORTKeyPress(TObject *Sender, wchar_t &Key)
{
    if (Key > '9' || Key < '0')
    {
        if (Key != VK_BACK && Key != VK_TAB && Key != '-')
            Key = 0;
    }
}
//---------------------------------------------------------------------------

void __fastcall TCRSM100M::Se_VDS_CHNLPropertiesChange(TObject *Sender)
{
    if (((TcxSpinEdit*)Sender)->Value < ((TcxSpinEdit*)Sender)->Properties->MinValue)
        ((TcxSpinEdit*)Sender)->Value = ((TcxSpinEdit*)Sender)->Properties->MinValue;
    else if(((TcxSpinEdit*)Sender)->Value > ((TcxSpinEdit*)Sender)->Properties->MaxValue &&
        ((TcxSpinEdit*)Sender)->Properties->MaxValue)
        ((TcxSpinEdit*)Sender)->Value = ((TcxSpinEdit*)Sender)->Properties->MaxValue;
}
//---------------------------------------------------------------------------
void __fastcall TCRSM100M::OpenMapCallback(TObject *ASender, const String AFunc, const String AArgs, const bool AFirst)
{
    try
    {
        if (AFirst)
        {
            //지도를 편집모드로 변경
            sJs.printf(L"setEditMode(%d, 1)", FLayerIdx);
            FRAMEOpenMap1->ExecScript(sJs);

            //시설물아이콘 등록
            UpdateIcon();

            sJs.printf(L"extentLayer(%d)", FLayerIdx);
            FRAMEOpenMap1->ExecScript(sJs);
            return;
        }

        if (AFunc.Pos("onfacilitydragend"))
        {
            TStringList *pStringList = new TStringList;
            pStringList->CommaText = AArgs;

            try
            {
                String sNmbr      = pStringList->Strings[0];
                String sCenterLat = pStringList->Strings[1];
                String sCenterLng = pStringList->Strings[2];
                SePosX->Value = sCenterLat.ToDouble();
                SePosY->Value = sCenterLng.ToDouble();
            }
            catch(Exception &e)
            {
            }
            delete pStringList;
        }
    }
    catch(Exception &exception)
    {
    }
}
//---------------------------------------------------------------------------

